.TH OKit 3 "May 2001" Unix Libraries

.SH NAME

OKit \- Object Tree, Virtual Machine and Compiler for concatenative languages in a shared library.

.SH DESCRIPTION

The
.B OKit
library provides an implementation of many basic data types, as integers,
floats, strings, lists, as well as executable primitives, encapsulated in a generic
object structure. Lists can contain themselves, as any of these primitive types.
Also included are methods for parsing and executing such lists, and error tracking.
The library as a whole provides the necessary framework for manipulating concatenative
code.
.
The object hierarchy tree is described below, stating object name, content, category
(between brackets), and string representation (between quotes).

.PD 0
.HP 0
.B
Object
		Generic object
.B
			  [none]
.P
							---
.P
.I
	Void
	Undefined object
.B
			  [null]
.P
							''
.P
.I
	Intg
	Integer number on 32 bits
.B
	   [scal]
.P
							'123456'
.P
.I
	Fltp
	Floating point number
.B
		  [scal]
.P
							'3.1415926e+00'
.P
.I
	Strg
	Character string
.B
			  [scal]
.P
							'"any text"'
.P
.I
	List
	All-purposes list
.B
			 [list]
.P
							'[ obj1 obj2 \.\.\. ]'
.P
.I
	Code
	Dynamic executable code
.B
		[exec]
.P
							'<CODE_NAME>' or '<CODE_NAME:data>'
.P
.I
	Bltn
	Static executable code
.B
		 [exec]
.P
							'BLTN_NAME'
.PD

.TP
Note 1:
Objects in a List can be separated by spaces, tabs, or linefeeds / carry returns

.TP
Note 2:
Here is a table of numeric values for special characters:
.PD 0
.P
		CHR	DEC	HEX			CHR	DEC	HEX
.P
		"	 34	0x22			:	 58	0x3A
.P
		<	 60	0x3C			>	 62	0x3E
.P
		[	 91	0x5B			]	 93	0x5D
.P
			171	0xAB				172	0xAC
.P
			187	0xBB
.PD



.SS Execution

"ListEval" or "ObjectEval" respectively take a List or an Object in addition
to a Stack and NameSpace, on which they will execute the List or Object.

The List type can contain any particular Object. It also provides a mecanism
to build a program, since when a List gets executed, all its elements are
executed in turn.
All Code and Bltn executable instructions, as well as the execution of a List,
use a Stack and a NameSpace. They take arguments and return results on the
Stack, while having access to named variables in the NameSpace. Execution of
a List is the act to take every member of this list and deposit it on the
Stack if it is a Void, Intg, Strg, or List, or execute it if it is a Code or
a Bltn.
Execution stops when the end of the list is reached, or the execution of an
[exec] type failed, from lack of arguments, bad argument types, or some other
error.

These execution routines return an error Status, that is an error code and the
associated message, providing a brief description of the error. <0,""> stands
for "no error".


.SS Compiler

"ListParse" or "ObjectParse" respectively take a character string and produce
a List or an Object. The string is interpretated into objects so that printing
these objects would produce back the original string, or something very similar.



.SS Syntax

A valid string for "ListParse" is one or more string representation of objects,
among:
.PD 0
.P
.I
	Built-ins :
	e.g. ' HELP ', a builtin command name
.P
.I
	Numbers   :
	e.g. ' 3.1425e+00 ', an integer or floating point number
.P
.I
	Strings   :
	e.g. ' "HELLO" ', a litteral string
.P
.I
	Lists     :
	e.g. ' [ obj1 obj2 ] ', a list of other objects
.PD


During parsing, a bareword is looked-up as a number, then if not applicable in
the set of built-in commands, and finally parsed as a string if it could not be
found.



.SS Data

The memory representation of an
.B Object
has two parts: a generic memory and tracking structure, used for reference counting
and cross-referencing, and a content part, type-dependant.

The
.B Void
data type is the default type: it is only used to mark internal
.B Object
use, or in a standard context that something has gone wrong in memory management.

The three scalar data types are simple counted memory segments: fixed size for the
.B Intg
and
.B Fltp
types, variable size (which means keeping a
.I 'size'
register) for the
.B Strg
type.

The
.B List
type is a doubly-linked list, each of its nodes (possibly non-existant) containing
an Object.

As for the
.B Code
and
.B Bltn
(executables) types, they are complex structures, containing, among other things,
a pointer toward a segment of assembler code retaining the actual implementation,
input and output arity information, and a formatted description.

The "system" types (
.B Stack
and
.B NameSpace,
or named variables table) play an essential role in memory management: Objects
can be freed when no reference for them in one of these tables exist any more.
They also are the essential holders for instruction arguments and results.




.SH BUGS

Innumerable. Don't forget to report them, even if each bug correction is the source
for new ones...




.SH TODO

Implement pseudo-code (list-compiled code) file dumping, in a cross-platform format.




.SH AUTHOR

.B	Author   :
	Yann LANDRIN-SCHWEITZER
.P
.B	Contact  :
	varkhan@free.fr
.P
.B	Homepage :
	http://varkhan.free.fr/



.SH SEE ALSO

.TP
.B OKeval:
.I [OKstrevl, OKbatchevl]
executable wrapper, and standard built-ins implementation, enabeling the
execution of arbitrary RPL code using these built-ins.




.SH LICENSE

This library is free software;  you can redistribute it and/or modify it
under  the terms of the  GNU Lesser General Public License  as published
by the  Free Software Foundation;  either version 2.1 of the License, or
(at your option) any later version.


This  library  is  distributed  in  the hope that it will be useful, but
WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
General Public License for more details.


You should have received a copy of the GNU Lesser General Public License
along with this library;  if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


